package travesty

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file t_container.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
(* This file is part of 'travesty'.

   Copyright (c) 2018 by Matt Windsor

   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
   files (the "Software"), to deal in the Software without
   restriction, including without limitation the rights to use, copy,
   modify, merge, publish, distribute, sublicense, and/or sell copies
   of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
   BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
   ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE. *)

open Core_kernel

include T_container_intf

let too_few_error () =
  Error.of_string "Expected one element; got none"
;;

let too_many_error _a =
  Container.Continue_or_stop.Stop
    ( Or_error.error_string
        "Expected one element; got too many"
    )
;;

module Extend0 (C : Container.S0)
  : Extensions0 with type t := C.t and type elt := C.elt = struct

  let max_measure ~measure ?(default=0) xs =
    xs
    |> C.max_elt ~compare:(T_fn.on measure Int.compare)
    |> Option.value_map ~f:measure ~default:default

  let at_most_one xs =
    C.fold_until xs
      ~init:`None_yet
      ~f:(function
          | `None_yet -> fun x -> Continue (`One x)
          | `One _    -> too_many_error

        )
      ~finish:(function
          | `None_yet -> Ok None
          | `One x    -> Ok (Some x)
        )
  ;;

  let one xs =
    Or_error.(
      xs
      |> at_most_one
      >>= Result.of_option ~error:(too_few_error ())
    )
  ;;

  let two xs =
    C.fold_until xs
      ~init:`None_yet
      ~f:(function
          | `None_yet -> fun x -> Continue (`One x)
          | `One x    -> fun y -> Continue (`Two (x, y))
          | `Two _    -> too_many_error
        )
      ~finish:(function
          | `None_yet | `One _ -> Result.Error (too_few_error ())
          | `Two (x, y)        -> Ok (x, y)
        )
  ;;
end

module Extend0_predicate (P : T) (C : Container.S0 with type elt = (P.t -> bool))
  : Extensions0_predicate with type t := C.t and type item := P.t = struct
  include Extend0 (C)

  let any  x ~predicates = C.exists predicates ~f:(fun p -> p x)
  let all  x ~predicates = C.for_all predicates ~f:(fun p -> p x)
  let none x ~predicates = not (any x ~predicates)
end

module Extend1 (C : Container.S1)
  : Extensions1 with type 'a t := 'a C.t = struct

  let max_measure ~measure ?(default=0) xs =
    xs
    |> C.max_elt ~compare:(T_fn.on measure Int.compare)
    |> Option.value_map ~f:measure ~default:default
  ;;

  let too_few_error () =
    Error.of_string "Expected one element; got none"
  ;;

  let too_many_error _a =
    Container.Continue_or_stop.Stop
      ( Or_error.error_string
          "Expected one element; got too many"
      )
  ;;

  let at_most_one xs =
    C.fold_until xs
      ~init:`None_yet
      ~f:(function
          | `None_yet -> fun x -> Continue (`One x)
          | `One _    -> too_many_error

        )
      ~finish:(function
          | `None_yet -> Ok None
          | `One x    -> Ok (Some x)
        )
  ;;

  let one xs =
    Or_error.(
      xs
      |> at_most_one
      >>= Result.of_option ~error:(too_few_error ())
    )
  ;;

  let two xs =
    C.fold_until xs
      ~init:`None_yet
      ~f:(function
          | `None_yet -> fun x -> Continue (`One x)
          | `One x    -> fun y -> Continue (`Two (x, y))
          | `Two _    -> too_many_error
        )
      ~finish:(function
          | `None_yet | `One _ -> Result.Error (too_few_error ())
          | `Two (x, y)        -> Ok (x, y)
        )
  ;;

  let any  x ~predicates = C.exists predicates ~f:(fun p -> p x)
  let all  x ~predicates = C.for_all predicates ~f:(fun p -> p x)
  let none x ~predicates = not (any x ~predicates)
end